package com.dh.fluttergooglefit;

import android.support.annotation.NonNull;
import android.util.Log;

import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;
import io.flutter.plugin.common.PluginRegistry.Registrar;
import io.flutter.plugin.common.PluginRegistry;

import com.google.android.gms.auth.api.signin.GoogleSignIn;
import com.google.android.gms.fitness.Fitness;
import com.google.android.gms.fitness.FitnessOptions;
import com.google.android.gms.fitness.data.DataPoint;
import com.google.android.gms.fitness.data.DataSet;
import com.google.android.gms.fitness.data.DataType;
import com.google.android.gms.fitness.data.Field;
import com.google.android.gms.fitness.request.DataReadRequest;
import com.google.android.gms.fitness.result.DataReadResponse;
import com.google.android.gms.tasks.OnCanceledListener;
import com.google.android.gms.tasks.OnCompleteListener;
import com.google.android.gms.tasks.OnFailureListener;
import com.google.android.gms.tasks.OnSuccessListener;
import com.google.android.gms.tasks.Task;

import java.util.Calendar;
import java.util.concurrent.TimeUnit;

public class FlutterGooglefitPlugin implements MethodCallHandler, PluginRegistry.RequestPermissionsResultListener {

  private Registrar registrar;
  private Result result;

  // 필요한 권한들 정의
  FitnessOptions fitnessOptions =
          FitnessOptions.builder()
                  .addDataType(DataType.TYPE_STEP_COUNT_CUMULATIVE)
                  .addDataType(DataType.TYPE_STEP_COUNT_DELTA)
                  .build();

  public static void registerWith(Registrar registrar) {
    final MethodChannel channel = new MethodChannel(registrar.messenger(), "com.dh/flutter_googlefit");
    FlutterGooglefitPlugin googlefitPlugin = new FlutterGooglefitPlugin(registrar);
    channel.setMethodCallHandler(googlefitPlugin);
    registrar.addRequestPermissionsResultListener(googlefitPlugin);
  }

  private FlutterGooglefitPlugin(Registrar registrar) {
    this.registrar = registrar;
  }

  @Override
  public void onMethodCall(MethodCall call, Result result) {

    switch (call.method) {
      case "reqPermissionGoogleFit":
        ReqPermissionGoogleFit();
        this.result = result;
        break;
      default:
        result.notImplemented();
        break;
    }
  }

  @Override
  public boolean onRequestPermissionsResult(int requestCode, String[] strings, int[] ints) {

    Log.e("tttttttttttttttt" , "tttttttttttttttttttttttttttttttttttttttttttttttttttttttttt : " + requestCode );
    if (requestCode == 0 && ints.length > 0) {
    }

    result.success("");
    return true;
  }


  // 피트 권한 확인.
  public boolean CheckPermissionFit()
  {
    return GoogleSignIn.hasPermissions(GoogleSignIn.getLastSignedInAccount(registrar.activity()), fitnessOptions);
  }

  // 피트 권한 요청.
  public void ReqPermissionGoogleFit()
  {
    if(!CheckPermissionFit()) {
      GoogleSignIn.requestPermissions(registrar.activity(), 375678, GoogleSignIn.getLastSignedInAccount(registrar.activity()), fitnessOptions);
    }
  }

  public void GetTotalStep()
  {
    if(CheckPermissionFit()){
      AddListenerFIt();
    }
  }

  public void AddListenerFIt()
  {
    Fitness.getRecordingClient(registrar.activity(),
            GoogleSignIn.getLastSignedInAccount(registrar.activity()))
            .subscribe(DataType.TYPE_STEP_COUNT_CUMULATIVE)
            .addOnFailureListener(new OnFailureListener() {
              @Override
              public void onFailure(@NonNull Exception e) {
                Log.i("healthcoin", "Failure listener");
              }
            })
            .addOnCompleteListener(new OnCompleteListener<Void>() {
              @Override
              public void onComplete(@NonNull Task<Void> task) {
                if (task.isSuccessful()) {
                  readTotalStepData();
                } else {
                  Log.w("healcoin", "There was a problem subscribing.", task.getException());
                }
              }
            });
  }

  public void readTotalStepData() {
    final Calendar cal = Calendar.getInstance();
    cal.add(Calendar.DATE, -1);
    // 시작 시간
    cal.set(cal.get(Calendar.YEAR), cal.get(Calendar.MONTH),
            cal.get(Calendar.DAY_OF_MONTH), 0, 0, 0);
    long startTime = cal.getTimeInMillis();

    // 종료 시간
    cal.set(cal.get(Calendar.YEAR), cal.get(Calendar.MONTH),
            cal.get(Calendar.DAY_OF_MONTH), 23, 59, 50);
    long endTime = cal.getTimeInMillis();

    Fitness.getHistoryClient(registrar.activity(),
            GoogleSignIn.getLastSignedInAccount(registrar.activity()))
            .readData(new DataReadRequest.Builder()
                    .read(DataType.AGGREGATE_STEP_COUNT_DELTA ) // Raw 걸음 수
                    .setTimeRange(startTime, endTime, TimeUnit.MILLISECONDS)
                    .build())
            .addOnFailureListener(new OnFailureListener() {
              @Override
              public void onFailure(@NonNull Exception e) {
                Log.i("healthcoin", "Failure readTotalStepData");
              }
            })
            .addOnCanceledListener(new OnCanceledListener() {
              @Override
              public void onCanceled() {
                Log.i("healthcoin", "Cancel readTotalStepData");
              }
            })
            .addOnSuccessListener(new OnSuccessListener<DataReadResponse>() {
              @Override
              public void onSuccess(DataReadResponse response) {
                DataSet dataSet = response.getDataSet(DataType.AGGREGATE_STEP_COUNT_DELTA );
                int temp = 0;
                for (DataPoint dp : dataSet.getDataPoints()) {
                  for (Field field : dp.getDataType().getFields()) {
                    temp = temp + Integer.parseInt( dp.getValue(field).toString() );
                  }
                }
                Log.i("healthcoin", "\tTotal Step : " + temp);

              }
            });
  }
}
